# 29/6/2015

##### The New Statistics with R, Chapter 11: Generalized Linear Mixed-effects Models

### Metadata
 # Source: Bolker et al. 2009 Trends in Ecology and Evolution 24, 127-135 (& supplement)
 # Note: X and rack are removed from simplified version 
 # X: Variable indexing each row in the dataset
 # reg:  Factor with  3 levels for the regions Sweden, Netherlands, Spain.
 # Popu: Factor with  9 levels for populations
 # gen:  Factor with 24 levels gives the genotypes
 # rack: Factor with  2 levels for the position in the greenhouse
 # nutrient: Fixed factor with  2 levels - 1 and 8.
 # amd: Fixed factor with  2 levels for apical meristem damage (applied or not).
 # total.fruits: Response variable; number of fruits (seed capsules) per plant. 

### Set up
# Clear workspace
rm(list=ls(all=TRUE)) 
# Turn off significance stars
options(show.signif.stars= FALSE) 
# set working directory (customise to your own settings)
# setwd(...)

### Packages
library(arm)    
library(ggplot2) 

######### Supplement: Loading and simplifying the orignal data file #########
# Note: For the New Stats analysis you can skip this chunk of code...
# Load original Bolker et al. data from csv file:
# Banta_TotalFruits.csv
dat.tf <- read.csv(file.choose(), header= TRUE) 
str(dat.tf)
# overwrite integer versions of gen, rack and nutrient with factor versions:
dat.tf$gen <- factor(dat.tf$gen)
dat.tf$rack <- factor(dat.tf$rack)
dat.tf$nutrient <- factor(dat.tf$nutrient)
# Make simplified version of data 
# dropping X and rack and adding full names
Banta <- dat.tf[ ,c(2,3,4,6,7,9) ]
names(Banta)
names(Banta) <- c("Region","Population","Genotype","Nutrient","Damage","Fruits")
# Set unclipped plants as intercept
Banta$Damage <- relevel(Banta$Damage, ref= "unclipped")
str(Banta)
# Create .txt file of simplified data:
write.table(Banta, file= "Banta.txt", sep="\t", col.names=TRUE, quote=FALSE, na="NA")
#####################################################################################

### Data
# Load data: NewStats_Chapter11_Data_Banta.txt
Banta <- read.table(file.choose(), header=TRUE)
# convert Genotype and Nutrient to factors:
Banta$Genotype <- factor(Banta$Genotype)
Banta$Nutrient <- factor(Banta$Nutrient)
# Box 11.1
str(Banta)

### 11.1: Model Building 

## Investigate random effects while retaining all fixed effects of interest

# This model fails to converge with R3.2.1 and concurrent lme4
# random effects for all genotype and population x treatment random effects.
gme1 <- glmer(Fruits~Nutrient*Damage
                    +(Nutrient*Damage|Population) +(Nutrient*Damage|Genotype),
                     family="poisson", data= Banta)              

############
# Box 11.2 #
############

# This model fails to converge with R3.2.1 and concurrent lme4
# Simplified model following selection in Bolker et al.
gme2 <- glmer(Fruits~Nutrient*Damage 
                    +(1|Population) +(Nutrient*Damage|Genotype),
                    family= "poisson", data= Banta)
 
# Model selection using AIC, BIC or likelihood-ratio test
anova(gme1, gme2) 
# AIC, BIC and LRT would all select simpler model

####### Supplement #########
# Model selection using DIC
d1 <- display(gme1)
d2 <- display(gme2)
( dic1 <- d1$DIC )
# 19091.37
( dic2 <- d2$DIC )
# 19091.37
# simpler model has a DIC ~9 units lower
########################################

############
# Box 11.3 #
############

# Point estimates for fixed effects
fixef(gme2)

############
# Box 11.3 #
############

# Estimates, SEs etc.
display(gme2)
summary(gme2) 
# confint(gme2) # only works for simpler models

################
# Box 11.4 & 5 #
################

# This model fails to converge with R3.2.1 and concurrent lme4
# Simplified model following selection in Bolker et al.
gme3 <- glmer(Fruits~Nutrient*Damage
                    +(Nutrient*Damage|Genotype),
                     family= "poisson", data= Banta)

# Model comparison
anova(gme2, gme3)

# Incomplete ANOVA table
anova(gme3)

### 11.4: Assumptions

# No way to check for overdispersion in glmm output!
# We can see the issue in plots of a GLM
glim <- glm(Fruits ~ Nutrient*Damage*Genotype + Population, data= Banta, family= "poisson")
par(mfrow=c(1,2))
plot(glim, which= c(1,2) )
#display(glim)
summary(glim)

##### Fig 9.8 #####
setEPS()
postscript("Fig11-1.eps")
par(mfrow=c(1,2))
plot(glim, which= c(1,2) )
dev.off()
#########

## Boxplots at the population level

# Damage effect across populations
qplot(Damage, log(Fruits+1), facets=Population~., data= Banta, geom= "boxplot",
      xlab= "Number of fruits (log+1)", ylab= "Apical Meristem Damage", main= "Damage by Population")+
      coord_flip()+theme_bw()

# Nutrient effect across populations
qplot(Nutrient, log(Fruits+1), facets=Population~., data= Banta, geom= "boxplot",
      xlab= "Number of fruits (log+1)", ylab= "Apical Meristem Damage", main= "Nutrients by Population")+
      coord_flip()+theme_bw()
