# 25/5/2015

##### The New Statistics with R: Chapter 2 #####

##### ANOVA of Darwin's Cross/Self Fertilized Maize

### Metadata
 # R package and dataframe: SMPracticals; darwin 
 # Source: Darwin 1876 'The effects of cross and self fertilisation in the vegetable kingdom'
 # 30 paired values of the 'fitness' (=height) of Maize seedlings
 # Aim: Estimating and testing effects of inbreeding (self- vs. cross-pollinated) on fitness (height)
 # pot: Blocking varaible for 4 pots (I-IV). Usually omitted from analysis. 
 # pair: Blocking variable for 15 pairs of maize seedlings
 # type: Explanatory factor with 2 levels - Cross- or Self-pollinated
 # height: Continuous response variable - Height of maize seedlings in inches

### Notes on R scripts
 # Metadata given at start of R scripts
 # Packages are loaded at the start (all together) and again individually at use
 # Code that may be needed only on first use of the script is 'turned off' using hashes
 # R code not printed in book is given as chunks of 'Supplementary R code'
 # Lines of code given inside outer parentheses print out their default output
 
### Optional set-up
 # library() function commands are given in the R script at first use
 # but all install() and library() functions are also given here as a 'start up' chunck
 # set working directory (customise to your own settings)
 # setwd(...)
 
### Install all packages used in this script:
 # only needs to be run at first use on any computer (therefore hashed off)
 # install.packages(SMPracticals, dependencies= TRUE) # install package from CRAN website
 # install.packages(reshape2, dependencies= TRUE)
 # install.packages(ggplot2, dependencies= TRUE)
 # install.packages(arm, dependencies= TRUE)

## Load all library packages used in this script:
library(SMPracticals) # Statistical Modelling Practicals package
library(ggplot2)      # Grammer of Graphics package, v.2
library(reshape)      # Reshape package
library(arm)          # Applied Regression Modelling package

# R script follows book from here

# install package from CRAN website
# install.packages(SMPracticals, dependencies= TRUE)

library(SMPracticals) # activates package for use in R
darwin # shows the data on screen

# Use ggplot2 package to plot height as a function of treatment
library(ggplot2) # activate package
qplot(type, height, data= darwin, shape= type, colour= type)+theme_bw()
# save the graph as a .eps file (infinitely scalable vector graphic)
ggsave(file= "Fig2-1.eps")
dev.off()

#################### Supplementary R code ############################
## Saving vector and raster graphics files
# Vector graphics are infinitely scalable
# create .eps version
qplot(type, height, data= darwin, shape= type, colour= type)+theme_bw()
ggsave(file= "Fig2-1.eps")
dev.off()
# Raster graphics have a set density of pixels (dpi)
# create .tiff version with lossless compression
qplot(type, height, data= darwin, shape= type, colour= type)+theme_bw()
# save the graph as a .tiff file (pixelated raster graphic)
# ggsave(file= "Fig2-1.tiff", dpi= 600)
dev.off()
#########

#################### Supplementary R code ############################
## A graph of the seedling heights with average values added:
 # Note the outer parentheses cause the graph to be displayed
( p <- qplot(type, height, data= darwin, shape= type, colour= type,
             main= "Individual and average seedling heights",
             ylab= "Seedling height (inches)",
             xlab= "Pollination")+theme_bw() )
# Create a function to calculate averages:
midm <- function(x) mean(x) 
# add averages to plot:
p + stat_summary( aes(colour= "Mean"), geom= "point", fun.y= midm)
# ggsave(file= "Fig2-1.eps")
dev.off()
#########

#################### Supplementary R code ############################
### How sum of squares works:
# Make a copy of the data re-ordered by type:
long <- darwin[ order(darwin$type), ]
# number the reordered data 1-30 for use in plotting
long$index <- c(1:30)
# postscript("Fig2-2.eps") # save graph as eps file
# landscape page with 3 panels
par(mfrow=c(1,3)) 
## SST
plot(long$height, pch= 16, main= "Square and sum differences for SST"
     , ylab= "Height", xlab= "Data ordered by type")
abline(mean(long$height), 0) # horizontal line at mean
for(i in 1:30) lines(c(i,i), c(mean(long$height), long$height[i]), lty= 3, col="darkgrey")
## SSA
long$predictions <- predict( lm(height ~ type, data= long) )
plot(long$height, type= "n", main= "Square and sum differences for SSA"
     , ylab= "Height", xlab= "Data ordered by type")
abline(mean(long$height), 0) # line at mean with slope of zero
points(long$height[long$type== "Cross"] ~ long$index[long$type== "Cross"], pch= 4, col= "darkgrey") 
points(long$height[long$type== "Self"] ~ long$index[long$type== "Self"], pch= 1, col= "darkgrey") 
# predicted values for crossed 
points(long$predictions[long$type== "Cross"] ~ long$index[long$type== "Cross"], pch= 4, col= "red")
# predicted values for selfed 
points(long$predictions[long$type== "Self"] ~ long$index[long$type== "Self"], pch= 1)
for(i in 1:30) 
lines( c(i,i), c( mean(long$height), long$predictions[i] ), lty= 3, col= "darkgrey")
## SSE
( means <- tapply(long$height, long$type, mean) )
plot(long$height, type= "n", main= "Square and sum differences for SSE"
     , ylab= "Height", xlab= "Data ordered by type")
points(long$height[long$type== "Cross"] ~ long$index[long$type== "Cross"], pch= 4, col= "red") 
points(long$height[long$type== "Self"] ~ long$index[long$type== "Self"], pch= 1)
lines(c(1, 15),c(means[1], means[1]), col= "red" )
lines(c(16, 30), c(means[2], means[2]))
for (i in 1:15) lines(c(i,i), c(means[1], long$height[i]),col="darkgrey", lty= 3)
for (i in 16:30) lines(c(i,i), c(means[2], long$height[i]),col="darkgrey", lty= 3)
dev.off()
#########

# The overall mean and standard deviation
( mean(darwin$height) )
( sd(darwin$height) )

# The standard deviation is the square root of the variance
( var(darwin$height) )
( sqrt( var(darwin$height)) )

### boxplot
# to have a description of the data, median, lower and upper quartile containing 50% of the data,
# the whiskers containing 95% of the data and the ouliers
qplot(type, height, data= darwin, geom= "boxplot",
      main= "Darwin's Maize: Box-&-Whisker plot", 
      xlab= "Fertilisation", ylab= "Height (inches)")+theme_bw()
# ggsave(file= "Fig2-3.eps")
dev.off()

# This 'long' format is what we need for linear model analysis like ANOVA
# and to produce some types of graph
# But, for some uses (e.g. t-tests) we need 'wide' format:
# Using melt() and cast() from Hadley Wickham's reshape2 package:
library(reshape)   
( mDarwin <- melt(darwin, id.vars= c("pot", "pair", "type"), measure.vars="height") ) 
# or in this case, simply:  melt(darwin)
# R reports: Using pot, pair, type as id variables
# the darwin dataframe rearranged in wide format using cast(): 
( darwide <- cast(mDarwin, pot + pair ~ variable + type) )

#################### Supplementary R code ############################
# Alternatively using the reshape FUNCTION
# darwide <- reshape(darwin, timevar= "type", idvar= "pair", v.names= "height", direction= "wide")
# Means for the two groups
( mean(darwide$height_Cross) )
( mean(darwide$height_Self) )
# Standard deviations for the two groups
( sd(darwide$height_Cross) )
( sd(darwide$height_Self) )
###########################

# ANOVA using Linear model function:

# Intercept only - in this case the grand mean
ls0 <- lm( height ~ 1, data= darwin) 
library(arm) # for the display() function
display(ls0)
# sd is the square root of the variance
( 3.18^2)
# 10.1124

# Linear model including pollination type
ls1 <- lm( height ~ 1+type, data= darwin)

display(ls1) # note that now the intercept is type=Self

# Use relevel() to set type=Self to be the intercept and refit the model: 
darwin$type <- relevel(darwin$type, ref= "Self")

# ANOVA table output
anova(ls1) 

# turn significance level stars off:
options(show.signif.stars= FALSE) 
summary(ls1)

# F = t^2 (for simple situations only)
(2.437^2)

# the probability of the F value if H0 true:
pf(5.9395, 1, 28, lower.tail= FALSE) 

# Two-way ANOVA including the pair blocking term
anova( lm( height ~ pair + type, data= darwin) )
# Does the answer change?
# Re-run the extractor functions from the last few lines...
# confint( lm( height ~ pair + type, data= darwin) )
# In this case pairing has an F value <1 ('negative variance component')
# this may indicate something wrong with the dataset
# for example, it is not clear if Darwin used random allocation of treatments to plants
# but this can also be just due to sampling 
# In any case, the P value for the paired anova is slightly larger than the unpaired
# but the outcome remains the same

### End of R script