# 30/4/2015

##### The New Statistics with R, Chapter 4: Linear Regression

### Metadata
 # Data from Williams 1959; Analysis by Bill (William) Venables with input from John Nelder
 # R package: SemiPar
 # 36 paired values of wood density and timber hardness ('Janka' scale)
 # dens (Density): Wood density in pounds per cubic foot
 # Hardness: Janka scale (pounds force required to push a ball bearing into wood sample)
 # Aim: Prediction of hardness from density
 
### Clear workspace and set options
rm(list=ls(all=TRUE)) 
# Turn off significance stars
options(show.signif.stars= FALSE) 
# set working directory (customise to your own settings)
# setwd("/Users/andrew/Documents/Writing/Books/HectorOUP/HectorOUP_Working")

### Packages: Install packages used (requires internet connection to CRAN)
# install.packages("SemiPar") # etc.

### Load all packages used:
library(SemiPar) # data
library(ggplot2) # graphics
library(MASS)    # Box-Cox
library(arm)     # display() etc.
library(grid)    # grid graphics

### Box 4.1:
library(SemiPar) # load package
# try(data(package= "SemiPar") ) # list datasets
data(janka) # load data
janka
names(janka) <- c("Density", "Hardness") # assign full names
janka 
## Box 4.1

## Normal least squares linear regression using lm() : 
janka.ls1 <- lm( Hardness~Density, data= janka )

########## Fig. 4.1: Regression with 95% CI #########
# Create x and y axis label objects with short names:
# quick to edit when same labels reused in many figures
xla <- expression(paste("Wood density (lbs " ,ft^-3, ")"))
yla <- "Hardness (pounds-force (lbf))"
( qplot(x= Density, y= Hardness, data= janka,
        geom= c("point","smooth"), method= "lm")+theme_bw()+
        xlab(xla)+ylab(yla) )
# save the graph as a raster graphic
# ggsave(file= "Fig4-1.tiff")
dev.off()
#########

## Prediction
# to predict hardness from density...
# y = a + bx, or equivalently, a*1 + b*x
# where a = intercept and b = slope, extracted:  
coef(janka.ls1) 

# model matrix holds the column of ones and the density values, x:
model.matrix(janka.ls1)

# e.g., predicted hardness for density = 24.7:
coef(janka.ls1)[1]*1 + coef(janka.ls1)[2]*24.7

# fitted values
fitted(janka.ls1)

# residuals
residuals(janka.ls1) # or: resid(janka.ls1)

### Plotting confidence and prediction intervals for lm models
# Generate a sequence of plenty of (100) densities to predict points for a smooth curve:
(xseq <- seq( min(janka$Density), max(janka$Density), length= 100 ) )
 
# Substitute this sequence for the observed densities
(predictions <- predict(janka.ls1, list(Density= xseq), se= TRUE ) )
# residual.scale is the residual standard deviation...
# the square root of the residual variance

anova(janka.ls1)
sqrt(33511) 
# [1] 183.0601

# t for P<0.05 & N=34 is very close to the large sample approximate value of 2
qt( 0.975, 34 ) # t for P<0.05 & N=34 = 2.032245

# An approximate 95% confidence interval (assuming t~2) 
up.ci <- predictions$fit + 2*predictions$se.fit # upper bound of 95% CI
lo.ci <- predictions$fit - 2*predictions$se.fit # lower bound of 95% CI

## predict() generates the limits if the interval is specified

# confidence interval limits from predict()
(ci.predictions <- predict(janka.ls1, list(Density= xseq), interval= "confidence", se= TRUE ) )

## Figure 4.2: Regression with 95% CI and PI using predict()
# postscript("Fig4-2.eps") # save graph as eps file
par(mfrow= c(1,2) ) # divide graphics device 'page' into 1 x 2 frame
# Figure of regression with 95% CI
plot(janka$Density, janka$Hardness,
     main= "95% Confidence Interval", 
     xlab= "Density", ylab= "Hardness" ) 
lines(ci.predictions$fit[,1]~xseq )        # regression line
lines(ci.predictions$fit[,2]~xseq, lty= 2) # 95% CI upper bound
lines(ci.predictions$fit[,3]~xseq, lty= 2) # 95% CI lower bound
# Figure of regression with 95% PI
plot(janka$Density, janka$Hardness,
     main= "95% Prediction Interval", 
     xlab= "Density", ylab= "Hardness" ) 
lines(pi.predictions$fit[,1]~xseq )        # regression line
lines(pi.predictions$fit[,2]~xseq, lty= 2) # 95% PI upper bound
lines(pi.predictions$fit[,3]~xseq, lty= 2) # 95% PI lower bound
dev.off()
#########
  
############# Fig 4.3 ############
# R's default model checking plots
par(mfrow= c(1,2) ) # divide graphics device 'page' into 2 x 2 frame
# postscript("Fig4-3.eps") # save graph as eps file
plot( janka.ls1, which= c(1:2) )
dev.off()
#########

# two main problems: 
# 1. residuals increase with the fitted values 
# 2. unexplained curvature 
# less importantly, a few points lie off the normal line in the qqplot 

# Square root transformation 
janka.sqrt <- lm(sqrt(Hardness)~Density, data= janka ) 
# model checking
# par(mfrow= c(2,2) )
# plot( janka.sqrt )
 
############# Fig 4.4 #############
# linearity is improved (no residual curvature)
(Fig4.4a <- qplot(Density, sqrt(Hardness), data= janka, 
                  geom= c("point","smooth"), method= "lm", formula= y~x,
                  main= "Square-root transformation")+theme_bw() )
dev.off()
# But variance still increases with the mean
# qplot of residuals vs fitted (predicted) values:
(Fig4.4b <- qplot(fitted(janka.sqrt), resid(janka.sqrt), ylab= "Residuals", xlab= "Predicted values",
                  main= "Variance increases with mean")+theme_bw()+
                  geom_hline(yintercept = 0, colour = "grey50") )
dev.off()
# Combine qplots into a side by side layout: 
## Fig 4.4
# library(grid)
# tiff("Fig4-4.tiff", width= 10, height= 5, units= 'in', res= 600)
# create viewport with a 1 row by 2 column layout:
pushViewport(viewport(layout=grid.layout(1,2) ) )
vplayout <- function(x, y)
viewport(layout.pos.row=x, layout.pos.col=y)
# print a on the left and b on the right: 
print(Fig4.4a, vp= vplayout(1,1))
print(Fig4.4b, vp= vplayout(1,2))
dev.off()
#########

# Transformation that gives the best straight line does not give constant variance. 

# Try a stronger transformation by taking the log of hardness: 
janka.log <- lm(log(Hardness)~Density, data= janka )
# par(mfrow= c(2,2) )
# plot( janka.log ) 

###################### Fig 4.5 ######################
(Fig4.5a <- qplot(Density, log(Hardness), data= janka, 
                   geom= c("point","smooth"), method= "lm",
                   main= "Log-transformed data")+theme_bw() )
dev.off()
# But variance still increases with the mean
# qplot of residuals vs fitted (predicted) values:
(Fig4.5b <- qplot(fitted(janka.log), resid(janka.log), ylab= "Residuals", xlab= "Predicted values",
                   main= "Unexplained curvature")+theme_bw()+
                   geom_hline(yintercept = 0, colour = "grey50") )
dev.off()                  
# Combine qplots into a side by side layout: 
# library(grid)
# tiff("Fig4-5.tiff", width= 10, height= 5, units= 'in', res= 600)
# create viewport with a 1 row by 2 column layout:
grid.newpage()
pushViewport(viewport(layout=grid.layout(1,2) ) )
vplayout <- function(x, y)
viewport(layout.pos.row=x, layout.pos.col=y)
# print a on the left and b on the right: 
print(Fig4.5a, vp= vplayout(1,1))
print(Fig4.5b, vp= vplayout(1,2))
dev.off()
#########

# Log transformation makes the variance approximately constant 
# (and improves normality slightly). 
# However, the locally weighted regression line still indicates curvature. 
# This could be modelled by including a quadratic term. 

#################### Supplementary R code ###################################
# Quadratic regression
janka.log.quad <- lm( log( Hardness ) ~ Density + I(Density^2), data= janka )
par( mfrow= c( 2,2 ) )
plot( janka.log.quad ) 
qplot(Density, sqrt(Hardness), data= janka, 
      geom= c("point","smooth"), method= "lm", formula= y~poly(x, 2),
      main= "Polynomial regression") + theme_bw()
dev.off()
#################### End of Supplementary R code ############################
