# 25/5/2015

##### The New Statistics with R, Chapter 5: Estimates and intervals 

### Set up
# Clear workspace
rm(list=ls(all=TRUE)) 
# set working directory (customise to your own settings)
# setwd("/.../...")

### Load all packages used:
library(SMPracticals) # Statistical Modelling Practicals package
library(ggplot2)      # Grammer of Graphics package
library(grid)         # To plot graphs in several windows
library(arm)          # for display and other functions

# darwin dataframe:
library(SMPracticals)
darwin

## 1. Raw data
p <- qplot(type, height, data= darwin, main= "Raw data", ylim=c(11,27) ) 
(a <- p + theme_bw() )
dev.off()

## 2. boxplots
p <- qplot(type, height, ylim=c(11,27), data= darwin, geom= "Boxplot", main= "Box plot")
(b <- p + theme_bw() )
dev.off()

### Box 5.1 ###
# linear model (unpaired analysis for simplicity)
lm1 <- lm(height ~ type, data= darwin)
# or, paired analysis:
#lm1 <- lm(height ~ pair + type, data= darwin)
# Create a data frame 'estimates' to hold...
# the means, SDs, SEMs or interval bounds:
estimates <- expand.grid(type=levels(darwin$type))
# Add the mean heights to the dataframe
estimates$height  <- predict(lm1, newdata= estimates)
# alternatively, mean heights using tapply()
#estimates$height  <- tapply(darwin$height, darwin$type, mean)
##############################################################

## 3. Means with SDs (pooled residual SD from diplay or summary)
display(lm(height ~ type, data= darwin)) # "residual sd = 2.94"
# the lm residual SD is called 'sigma.hat' and can be extracted like this
lmDisplay <- display(lm(height ~ type, data= darwin))
lmDisplay$sigma.hat
( estimates$sd  <- rep(lmDisplay$sigma.hat, 2) )
# Plot of means with SDs
p <- qplot(type, height, ylim=c(11,27), data=estimates, main="Std Deviations")
(c <- p  + geom_linerange(aes(min=height-sd, max=height+sd)) + theme_bw() )
dev.off()

## 4. Means with SEMs
# add SEMs to estimates dataframe
(estimates$sem  <- predict(lm1, newdata= estimates, se.fit= TRUE)$se.fit )
# Plot of means with SEMs
p <- qplot(type, height, ylim=c(11,27), data=estimates, main="Std Errors")
(d <- p + geom_linerange(aes(min=height-sem, max=height+sem)) + theme_bw() )
dev.off()

## 5. Means with 95% CIs
# add 95% CI upper and lower bounds to estimates dataframe
estimates$C95_lwr  <- predict(lm1, newdata= estimates, interval="confidence")[,2]
estimates$C95_upr  <- predict(lm1, newdata= estimates, interval="confidence")[,3]
# Plot of means with 95% CIs
p <- qplot(type, height, ylim=c(11,27), data=estimates, main="95% CIs")
(e <- p + geom_linerange(aes(min=C95_lwr, max=C95_upr)) + theme_bw() )
# ggsave(file= "Fig3-3.eps")
dev.off()

## 6. [Box 5.2] Means with a range of CIs (+/- 2 and 3 SEMs)
p <- qplot(type, height, ylim=c(11,27), data=estimates, main="Multiple CIs")
(f <- p + theme_bw() + geom_point(size=2) +
#geom_linerange(aes(min=height-sem, max=height+sem),size=2) +
geom_linerange(aes(min=height-2*sem, max=height+2*sem),size=0.7) +
geom_linerange(aes(min=height-3*sem, max=height+3*sem),size=0.3) )
dev.off()

## 7. Means with LSDs
# add Standard Error of the Difference between means to estimates dataframe
(estimates$sed  <- summary(lm1)$coefficients[2,2] )
# Plot of means with LSDs (P<0.05)
p <- qplot(type, height, ylim=c(11,27), data= estimates, main="LSDs")
(g <- p + theme_bw() + geom_linerange(aes(min=height-sed, max=height+sed)) )
# approximating as 2*SED (or use t = 2.14; n=14; i.e. not +/-1 SED but +/-1.07 SED)
# ggsave(file= "Fig3-4.eps")
dev.off()

## 8. Means with Prediction intervals
# add 95% Prediction Interval upper and lower bounds to dataframe
estimates$pred_lwr  <- predict(lm1, newdata= estimates, interval= "prediction", level= 0.95)[,2]
estimates$pred_upr  <- predict(lm1, newdata= estimates, interval= "prediction", level= 0.95)[,3]
estimates
# 95% Prediction Intervals
p <- qplot(type, height, ylim=c(11,27), data= estimates, main= "95% PIs")
(h <- p + geom_linerange(aes(min=pred_lwr, max=pred_upr)) + theme_bw() )
dev.off()

### Fig. 5.1: Multipanel graph of descriptive approaches (landscape)
setEPS()
postscript("Fig5-1.eps")
grid.newpage()
pushViewport(viewport(layout = grid.layout(1, 3)))
vplayout <- function(x, y)
  viewport(layout.pos.row = x, layout.pos.col = y)
print(a, vp = vplayout(1, 1)) # Raw
print(b, vp = vplayout(1, 2)) # Boxplots
print(c, vp = vplayout(1, 3)) # SDs
dev.off()

### Fig. 5.2: Multipanel graph of inferential approaches (landscape)
setEPS()
postscript("Fig5-2.eps")
grid.newpage()
pushViewport(viewport(layout = grid.layout(2, 3)))
vplayout <- function(x, y)
  viewport(layout.pos.row = x, layout.pos.col = y)
print(d, vp = vplayout(1, 1)) # SEs
print(e, vp = vplayout(1, 2)) # 95% CIs
print(f, vp = vplayout(1, 3)) # Multiple (95 & 99%) CIs
print(g, vp = vplayout(2, 1)) # LSDs
print(h, vp = vplayout(2, 2)) # 95% Prediction interval
dev.off()

### Supplement: Multipanel graph of all intervals for comparison (portrait)
grid.newpage()
pushViewport(viewport(layout = grid.layout(4, 2)))
vplayout <- function(x, y)
  viewport(layout.pos.row = x, layout.pos.col = y)
print(a, vp = vplayout(1, 1)) # Raw
print(b, vp = vplayout(1, 2)) # Boxplots
print(c, vp = vplayout(2, 1)) # SDs
print(d, vp = vplayout(2, 2)) # SEs
print(e, vp = vplayout(3, 1)) # 95% CIs
print(f, vp = vplayout(3, 2)) # 95% CIs
print(g, vp = vplayout(4, 1)) # LSDs
print(h, vp = vplayout(4, 2)) # 95% Prediction interval
dev.off()
#########

### Supplement: Multipanel graph of all intervals for comparison (landscape)
grid.newpage()
pushViewport(viewport(layout = grid.layout(2, 4)))
vplayout <- function(x, y)
  viewport(layout.pos.row = x, layout.pos.col = y)
print(a, vp = vplayout(1, 1)) # Raw
print(b, vp = vplayout(1, 2)) # Boxplots
print(c, vp = vplayout(1, 3)) # SDs
print(d, vp = vplayout(1, 4)) # SEs
print(e, vp = vplayout(2, 1)) # 95% CIs
print(f, vp = vplayout(2, 2)) # 95% CIs
print(g, vp = vplayout(2, 3)) # LSDs
print(h, vp = vplayout(2, 4)) # 95% Prediction interval
dev.off()
#########

### Fig. 5.3: Relating different types of interval and error bar
# Dataframe
Treatment <- c("Control", "Treated")
means1 <- c(3, 4)
means2 <- c(3, 5.8)
means3 <- c(3, 7)
sem       <- 1
(dframe <- data.frame(Treatment, means1, means2, means3, sem) )
# A. Non-significant
(p <- qplot(Treatment, means1, data= dframe, main="P > 0.05", xlab= "", ylab= "Response", ylim= c(1,9)) )
(a <- p + theme_bw() + geom_point(size=2) +
geom_linerange(aes(min= means1-1.4*sem, max= means1+1.4*sem),size= 1) +
geom_linerange(aes(min= means1-2*sem, max= means1+2*sem), size= 0.5) )
# B. Significant
(p <- qplot(Treatment, means3, data= dframe, main="P ~ 0.01", xlab= "", ylab= "", ylim= c(1,9)) )
(b <- p + theme_bw() + geom_point(size=2) +
geom_linerange(aes(min= means3-1.4*sem, max= means3+1.4*sem),size= 1) +
geom_linerange(aes(min= means3-2*sem, max= means3+2*sem), size= 0.5) )
# C. Significant
(p <- qplot(Treatment, means2, data= dframe, main="P ~ 0.05", xlab= "", ylab= "", ylim= c(1,9)) )
(c <- p + theme_bw() + geom_point(size=2) +
geom_linerange(aes(min= means2-1.4*sem, max= means2+1.4*sem),size= 1) +
geom_linerange(aes(min= means2-2*sem, max= means2+2*sem), size= 0.5) )
dev.off()

## Multipanel graph (landscape)
setEPS()
postscript("Fig5-3.eps")
grid.newpage()
pushViewport(viewport(layout = grid.layout(1, 3)))
vplayout <- function(x, y)
  viewport(layout.pos.row = x, layout.pos.col = y)
print(a, vp = vplayout(1, 1)) # NS
print(b, vp = vplayout(1, 2)) # P ~ 0.01
print(c, vp = vplayout(1, 3)) # P ~ 0.05
dev.off()

### Fig. 5.4: Interpreting intervals
# Dataframe and graph of different scenarios
Scenario <- c("A","B","C")
Response <- c(1,8,8)
MOE <- c(3,3,10)
(frame <- data.frame(Scenario, Response, MOE) )
p <- qplot(Scenario, Response, data= frame, main="Confidence Interval Scenarios")
(p2 <- p  + geom_linerange(aes(min= Response-MOE, max= Response+MOE)) + theme_bw() )
ggsave("Fig5.4.eps")
dev.off()

###  Fig. 5.5: Synthesizing results
Study <- c("Darwin","Wallace")
Effect <- c(2.6, 2.2)
SEM <- c(1.2, 1.2)
(d.frame <- data.frame(Study, Effect, SEM) )
p <- qplot(Study, Effect, data= d.frame, main="Effect of out-crossing")
(p2 <- p  + geom_linerange(aes(min= Effect-2*SEM, max= Effect+2*SEM)) + theme_bw() )
ggsave("Fig5.5.pdf")
ggsave("Fig5.5.eps")
dev.off()

###########
# Box 5.3 #
###########